@@ -315,7 +315,7 @@ gap <- function(variable, data, groupA = "default", groupB = "default",
                 returnSimpleN=FALSE,
                 returnNumberOfPSU=FALSE,
                 noCov=FALSE,
-                pctMethod=c("unbiased", "simple"),
+                pctMethod=c("unbiased", "symmetric", "simple"),
                 includeLinkingError=FALSE) {
   if(is.character(substitute(groupA))) {
     groupA <- parse(text=groupA)[[1]]
@@ -359,8 +359,8 @@ gap <- function(variable, data, groupA = "default", groupB = "default",
   
   # check if linking error is supported for this survey
   if(!missing(includeLinkingError) && includeLinkingError) {
-    checkDataClass(data, "edsurvey.data.frame.list")
-    if(!c("NAEP", "PISA") %in% survey) {
+    checkDataClass(data, c("edsurvey.data.frame.list"))
+    if(!any(c("NAEP", "PISA") %in% survey)) {
       stop("the argument ", dQuote("includeLinkingError"), " can only be set to ", dQuote("TRUE"), " when EdSurvey suports linking error for the survey. Currently EdSurvey supports linking error for NAEP and PISA.")
     }
   }
@@ -384,14 +384,16 @@ gap <- function(variable, data, groupA = "default", groupB = "default",
   if (!missing(includeLinkingError) && includeLinkingError)  {
     # the following is only for NAEP which has linking error within survey
     if("NAEP" %in% survey) {     
-      uniqueSubject <- unique(unlist(lapply(data$data, getAttributes, "subject")))
-      if (length(uniqueSubject) != 1) {
-        stop("Linking error only support ", dQuote("edsurvey.data.frame.list"), " with a single subject.")
-      }
+      if(inherits(data, "edsurvey.data.frame.list")) {
+        uniqueSubject <- unique(unlist(lapply(data$data, getAttributes, "subject")))
+        if (length(uniqueSubject) != 1) {
+          stop(paste0("Linking error only support ", dQuote("edsurvey.data.frame.list"), " with a single subject. Found subjects: ", pasteItems(uniqueSubject), "."))
+        }
       
-      uniqueGrade <- unique(unlist(lapply(data$data, getAttributes, "gradeLevel")))
-      if (length(uniqueGrade) != 1) {
-        stop("Linking error only support ", dQuote("edsurvey.data.frame.list"), " with a single grade level.")
+        uniqueGrade <- unique(unlist(lapply(data$data, getAttributes, "gradeLevel")))
+        if (length(uniqueGrade) != 1) {
+          stop(paste0("Linking error only support ", dQuote("edsurvey.data.frame.list"), " with a single grade level. Found grades: ", pasteItems(uniqueGrade), "."))
+        }
       }
         
       # For achievementDiscrete=TRUE, return "At basic" (not "below basic") if "achievementLevel = "basic"
@@ -425,11 +427,16 @@ gap <- function(variable, data, groupA = "default", groupB = "default",
         
       # Get DBA labels from DBA years
       # 2017 is the first NAEP DBA year
-      allYears <- unlist(itterateESDFL(call("getAttributes", attribute="year"), data))
-      dbaLabels <- data[[2]]$labels[allYears >= 2017]
-      if (!all(dbaLabels %in% data[[2]]$labels)) {
-        stop(paste0(pasteItems(dbaLabels[!dbaLabels %in% data[[2]]$labels]), 
-                    " is not in the ", dQuote("edsurvey.data.frame.list"), " labels."))
+      if(inherits(data, "edsurvey.data.frame.list")) {
+        allYears <- unlist(itterateESDFL(call("getAttributes", attribute="year"), data))
+        dbaLabels <- data[[2]]$labels[allYears >= 2017]
+        if (!all(dbaLabels %in% data[[2]]$labels)) {
+          stop(paste0(pasteItems(dbaLabels[!dbaLabels %in% data[[2]]$labels]), 
+                      " is not in the ", dQuote("edsurvey.data.frame.list"), " labels."))
+        }
+      } else {
+        allYears <- getAttributes(data, attribute="year")
+        dbaLabels <- NULL
       }
       if (!is.null(percentiles) && !all(percentiles %in% c(10, 25, 50, 75, 90))) {
         stop("NAEP linking error only supported for 10th, 25th, 50th, 75th, 90th percentiles.", 
@@ -613,7 +620,7 @@ gap <- function(variable, data, groupA = "default", groupB = "default",
             # if the datasets and from the same sample, add covariances and do DoFCorrection
             if(sameSurvey(data$datalist[[i]], data$datalist[[refi]])) {
               if (nrow(resi$varEstInputs$JK) != 0) { # some Level (percentile or achievementLevel) do not have varEstInput data so the subset
-              									   # in line 476 will return an empty data.frame
+                                   # in line 476 will return an empty data.frame
                 resdf$dofAA[i] <- DoFCorrection(resi$varEstInputs, refVarEstInputs, "A", method="JR")
               }
               resdf$covAA[i] <- varEstToCov(resi$varEstInputs, refVarEstInputs, "A", jkSumMultiplier=getAttributes(data$datalist[[i]], "jkSumMultiplier"))
@@ -899,7 +906,7 @@ gapHelper <- function(variable, data, groupA = "default", groupB = "default",
                       returnSimpleN=FALSE,
                       returnNumberOfPSU=FALSE,
                       noCov=FALSE,
-                      pctMethod=c("unbiased", "simple"),
+                      pctMethod=c("unbiased", "symmetric", "simple"),
                       varMethod=NULL,
                       includeLinkingError=NULL) {
   if(is.character(substitute(groupA))) {
@@ -1069,7 +1076,6 @@ gapHelper <- function(variable, data, groupA = "default", groupB = "default",
     } else {
       SEs <- list(estimateAse = meanA$se)
     }
-    
     maJK <- attributes(meanA)$varEstInputs$JK
     maJK$Level <- maJK$variable
     maJK$variable <- "A"
@@ -1122,9 +1128,21 @@ gapHelper <- function(variable, data, groupA = "default", groupB = "default",
     als <- getALNames(data, variable)
     callalA <- c(callal, list(data=dataA))
     meanA <- do.call(achievementLevels, callalA)
+    if(!is.null(meanA$discrete)) {
+      colnames(meanA$discrete)[grepl("Level$", colnames(meanA$discrete))] <- "Level"
+    }
+    if(!is.null(meanA$cumulative)) {
+      colnames(meanA$cumulative)[grepl("Level$", colnames(meanA$cumulative))] <- "Level"
+    }
     if (!skipB) {
       callalB <- c(callal, list(data=dataB))
       meanB <- do.call(achievementLevels, callalB)
+      if(!is.null(meanB$discrete)) {
+        colnames(meanB$discrete)[grepl("Level$", colnames(meanB$discrete))] <- "Level"
+      }
+      if(!is.null(meanB$cumulative)) {
+        colnames(meanB$cumulative)[grepl("Level$", colnames(meanB$cumulative))] <- "Level"
+      }
     }
     # achievementDiscrete indicates whether we should extract cumulative or discrete achievement level results
     if (achievementDiscrete){
@@ -1145,7 +1163,7 @@ gapHelper <- function(variable, data, groupA = "default", groupB = "default",
         if (typeof(als) == "character") {
           lB <- sapply(achievementLevel, function(al) {
             lal <- addALPrefix(al=al, als=als, discrete=TRUE)
-            grep(lal,meanB$discrete$Level, ignore.case = TRUE)
+            grep(lal, meanB$discrete$Level, ignore.case = TRUE)
           })} else {
             lB <- rep(TRUE, length(als[[achievementLevel]]))
           }
@@ -1426,16 +1444,16 @@ gapHelper <- function(variable, data, groupA = "default", groupB = "default",
             JRdf <- c(JRdf,DoFCorrection(varEstInputsTemp, varA="A", varB="B", method=c("JR")))   
           } # end if (!is.null(varEstInputsTemp$JK))
           else {
-          	cov <- c(cov,NA)
-          	JRdfA <- c(JRdfA,NA)
-          	JRdfB <- c(JRdfB,NA)
-          	JRdf <- c(JRdf,NA)
+          	cov <- c(cov, NA)
+          	JRdfA <- c(JRdfA, NA)
+          	JRdfB <- c(JRdfB, NA)
+          	JRdf <- c(JRdf, NA)
           }
         }
       } #end if (type %in% c("pct","AL")) 
       else {
-        cov <- ifelse(groupA_0 | groupB_0,cov,varEstToCov(varEstInputs, varA="A", varB="B", jkSumMultiplier=getAttributes(data, "jkSumMultiplier")))
-        JRdfA <- ifelse(groupA_0,NA,DoFCorrection(varEstInputs, varA="A", method=c("JR")))
+        cov <- ifelse(groupA_0 | groupB_0, cov, varEstToCov(varEstInputs, varA="A", varB="B", jkSumMultiplier=getAttributes(data, "jkSumMultiplier")))
+        JRdfA <- ifelse(groupA_0, NA, DoFCorrection(varEstInputs, varA="A", method=c("JR")))
         JRdfB <- ifelse(groupB_0, NA, DoFCorrection(varEstInputs, varA="B", method=c("JR")))
         JRdf <- ifelse(groupA_0 | groupB_0, NA, DoFCorrection(varEstInputs, varA="A", varB="B", method=c("JR"))) 
       }
