@@ -1,4 +1,5 @@
 # @author Trang Nguyen and Paul Bailey
+#' @importFrom utils find
 #' @method subset edsurvey.data.frame.list
 #' @export
 subset.edsurvey.data.frame.list <- function(x, subset, inside=FALSE, drop = FALSE, ...) {
@@ -23,45 +24,54 @@ subset.edsurvey.data.frame.list <- function(x, subset, inside=FALSE, drop = FALS
     # parse the subset
     # substitute in variables that are available in the current environment
     subset_call <- substitute(subset)
-    iparse <- function(ccall,env) {
+    iparse <- function(iparseCall, subsetEnv, iparseDepth=1) {
       # for each element
-      for(i in 1:length(ccall)) {
+      for(iparseind in 1:length(iparseCall)) {
         # if it is a name
-        if(class(ccall[[i]]) %in% c("name")) {
-          ccall_c <- as.character(ccall[[i]])
+        if(class(iparseCall[[iparseind]]) %in% c("name")) {
+          iparseCall_c <- as.character(iparseCall[[iparseind]])
           # if it is not in the data and is in the parent.frame, then substitue it now.
-          if((! ccall_c %in% unlist(colnames(x))) & (ccall_c %in% ls(envir=env)) ) {
-            if (ccall[[i]] == "%in%" || is.function(ccall[[i]])) {
-              ev <- eval(substitute(ccall[[i]]), parent.frame())  
-            } else {
-              ev <- eval(ccall[[i]], parent.frame())
-            } #end of if/esle statment: if ccall[[i]] == "%in%" || is.function(ccall[[i]])
-            ccall[[i]] <- ev
-          } # End of if statment: if (! ccall_c %in% colnames(x$data)) & (ccall_c %in% ls(envir=env)) 
-        } # end if(class(ccall[[i]]) %in% c("name")) {
-        if(class(ccall[[i]]) %in% "call") {
+          if(! iparseCall_c %in% colnames(x)) {
+            if(length(find(iparseCall_c)) > 0) {
+              if (iparseCall[[iparseind]] == "%in%" || is.function(iparseCall[[iparseind]]) || is.function(get(iparseCall_c, find(iparseCall_c)))) {
+                ev <- eval(substitute(iparseCall[[iparseind]]), parent.frame())
+              } else {
+                # get the variable
+                ev <- eval(iparseCall[[iparseind]], parent.frame())
+              }
+              iparseCall[[iparseind]] <- ev
+            } #end if length(find(iparseCall_c)) > 0)
+            # but, if dynGet returns, use that instead
+            iparsedg <- dynGet(iparseCall_c, ifnotfound="", minframe=1L)
+            # if dynGet found something
+            if(any(iparsedg != "")) {
+              iparseCall[[iparseind]] <- iparsedg
+            }
+          } # end if(!call_c)
+        } # end if(class(iparseCall[[iparseind]]) %in% c("name")) {
+        if(class(iparseCall[[iparseind]]) %in% "call") {
           # if this is a call, recursively parse that
-          ccall[[i]] <- iparse(ccall[[i]], env)
-        } #end of if statment: if class(ccall[[i]]) %in% "call"
-      } # end of for loop: i in 1:length(ccall)
-      ccall
+          iparseCall[[iparseind]] <- iparse(iparseCall[[iparseind]], subsetEnv, iparseDepth = iparseDepth + 1)
+        } #end of if statment: if class(iparseCall[[i]]) %in% "call"
+      } # end of for loop: i in 1:length(iparseCall)
+      iparseCall
     } # End of fucntion: iparse
     subset_call <- iparse(subset_call, parent.frame())
   } # Enf of if esle statmet: if inside is true 
 
   res <- x
   subsetVars <- all.vars(subset_call)
-  res$datalist <- lapply(1:length(x$datalist), function(i) {
-    li <- x$datalist[[i]]
+  res$datalist <- lapply(1:length(x$datalist), function(dataListi) {
+    dataList_li <- x$datalist[[dataListi]]
     # check whether the variable exists the edsurvey.data.frame
-    for (v in subsetVars) {
-      if (!v %in% colnames(li)) {
-        warning(paste0("Variable ", sQuote(v), "is not found in the data ",sQuote(x$covs[i,]),"."))
+    for (dataList_v in subsetVars) {
+      if (!dataList_v %in% colnames(dataList_li)) {
+        warning(paste0("Variable ", sQuote(dataList_v), "is not found in the data ",sQuote(x$covs[dataListi,]),"."))
         return(NULL)
       }
     }
-    li[["userConditions"]] <- c(li[["userConditions"]],list(subset_call))
-    li
+    dataList_li[["userConditions"]] <- c(dataList_li[["userConditions"]], list(subset_call))
+    dataList_li
   })
   
   # Remove NULL element
