@@ -1,5 +1,5 @@
 \dontrun{
-setwd("C:/B&B2001/ecbw/B01")
+setwd("~/B&B2001/ecbw/B01")
 
 dervFP <- file.path(getwd(), "Data", "B01DER.dat")
 wgtFP <- file.path(getwd(), "Data", "B01WT.dat")
