@@ -1,5 +1,7 @@
+\dontrun{
 # read in the example data (generated, not real student data)
 sdf <- readNAEP(system.file("extdata/data", "M36NT2PM.dat", package="NAEPprimer"))
 
 # show the cut points
 showCutPoints(data=sdf)
+}
